import numpy as np
import cv2
import matplotlib.pyplot as plt
from scipy import signal


def compute_light_percentage(image, thresh):
    # 计算亮度大于某个阈值的占比
    assert len(image.shape) == 2
    img_h, img_w = image.shape[:2]
    dark_area = (image >= thresh).sum()
    area = img_h * img_w
    return 1.0 * dark_area / area


def compute_dark_percentage(image, thresh):
    # 计算亮度黑于某个阈值的占比
    assert len(image.shape) == 2
    img_h, img_w = image.shape[:2]
    dark_area = (image <= thresh).sum()
    area = img_h * img_w
    return 1.0 * dark_area / area


def avg_filt_1d(vector, kernel_size):
    # 对向量进行卷积平滑操作
    """
    Args:
            vector (array): Input vector
            kernel_size (int): kernel size of average filtering
    Returns:
            vector (array): Smoothed vector
    """
    kernel = np.ones([kernel_size]) / kernel_size
    return np.convolve(vector, kernel, mode='same')


def to_gray(image):
    # 将图片转灰度图
    if len(image.shape) == 2:
        pass
    elif len(image.shape) == 3:
        image = cv2.cvtColor(image, cv2.COLOR_BGR2GRAY)
    else:
        raise Exception(('Image shape error: {}').format(image.shape))
    return image


def padding(image, pad_h, pad_w):
    # 给图片添加padding
    image_shape = np.array(image.shape)
    h, w = image_shape[:2]
    pad_shape = image_shape.copy()
    pad_shape[0] = pad_h
    pad_shape[1] = pad_w
    image_pad = np.zeros(pad_shape, dtype=image.dtype)
    image_pad[:h, :w, ...] = image
    return image_pad


def get_median(vector):
    # 获取向量的中位数
    assert len(vector) >= 1
    return np.sort(vector)[int(len(vector) / 2)]


def _listornparray_2_plt(row_histogram):
    """将list或者nparray画成图 画直方图"""
    lenth = len(row_histogram)
    plt.figure(1)
    hist_x = np.linspace(0, lenth - 1, lenth)
    plt.title("row_histogram")
    plt.rcParams['figure.figsize'] = (lenth, 8)  # 单位是inches

    x_major_locator = plt.MultipleLocator(1)
    ax = plt.gca()
    ax.xaxis.set_major_locator(x_major_locator)  # x轴按1刻度显示

    plt.plot(hist_x, row_histogram)
    higher_q = np.max(row_histogram) * (5 / 8)
    plt.plot([0, lenth - 1], [higher_q, higher_q])
    num_peak_3 = signal.find_peaks(row_histogram, distance=1)  # distance表极大值点的距离至少大于等于10个水平单位
    for ii in range(len(num_peak_3[0])):
        if (row_histogram[num_peak_3[0][ii]] > np.mean(row_histogram)) and (
                row_histogram[num_peak_3[0][ii]] != np.max(row_histogram)
        ) and (
                num_peak_3[0][ii] > 10
        ):
            plt.plot(num_peak_3[0][ii], row_histogram[num_peak_3[0][ii]], '*', markersize=10)
            plt.axvline(num_peak_3[0][ii])
            print(num_peak_3[0][ii])

    plt.savefig("row_histogram_peak.jpg")
    plt.close()


def graytoprofile(gray):
    # 边缘检测
    kernel = cv2.getStructuringElement(cv2.MORPH_RECT, (3, 3))
    dilate = cv2.dilate(gray, kernel)
    erode = cv2.erode(gray, kernel)
    profile = cv2.absdiff(dilate, erode)
    return profile


def sqrt_var_hist(histogram):
    # 求直方图的方差开根号
    return np.sqrt(np.var(histogram))


def remove_baseline_shift(vector, step=10):
    """
    Args:
            vector (array): Numpy vector with shape (num_points,)
            step (int): Kernel size of average filtering
    Returns:
            vector (array): Vector with baseline shift removed
    """
    if step % 2 == 0:
        step += 1
    vector_smoothed = signal.medfilt(vector, step)
    avg_smoothed = avg_filt_1d(vector, kernel_size=step)
    temp_step = int(len(vector) / 40)
    center_ind = int(len(vector) / 2)
    x1 = center_ind - temp_step
    x2 = center_ind + temp_step
    vector_smoothed[x1:x2] = avg_smoothed[x1:x2]
    return vector - vector_smoothed


def plt_show_cv2(image):
    # plt 显示 opencv 格式的图片
    plt.figure(1)
    if len(image.shape) == 2:
        ## 把单通道的灰度图转换成三通道的灰度图
        image = np.concatenate(
            (np.expand_dims(image, axis=2), np.expand_dims(image, axis=2), np.expand_dims(image, axis=2)), axis=-1)
        # plt.imshow(image,cmap='gray',interpolation='bicubic')#显示灰度图
    else:
        image = cv2.cvtColor(image, cv2.COLOR_BGR2RGB)
    plt.imshow(image)
    plt.show()


def crop_image_like(image, crop_bbox):
    """
    获取bbox对应的图片
    :param image:
    :param crop_bbox:
    :return:
    """
    xmin, ymin, xmax, ymax = crop_bbox
    if xmax == -1:
        xmax = image.shape[1] + 1
    if ymax == -1:
        ymax = image.shape[0] + 1
    image_crop = image[ymin:ymax + 1, xmin:xmax + 1, ...]
    return image_crop

"""
滤波器
left_rectangle = cv2.blur(left_rectangle, (3, 3))
left_rectangle = cv2.GaussianBlur(left_rectangle, (5, 5), 0)
left_rectangle = cv2.medianBlur(left_rectangle, 3)
left_rectangle = cv2.bilateralFilter(left_rectangle, 9, 75, 75)

求突然上升很大值和突然下降很大值的位置
col_histogram = np.sum(subImg2, axis=0)
nInteral = 20
cols_minInterval = subImg2.shape[1] // nInteral
## left
left_scope_diff = [np.sum(col_histogram[idx - nInteral:idx].astype(np.int)) - np.sum(
    col_histogram[idx:idx + nInteral].astype(np.int))
                   for idx in range(nInteral, cols_minInterval - nInteral)]
left_edge = nInteral + np.argmax(left_scope_diff)
## right
right_scope_diff = [np.sum(col_histogram[idx - nInteral:idx].astype(np.int)) - np.sum(
    col_histogram[idx:idx + nInteral].astype(np.int))
                    for idx in range(subImg2.shape[1] - cols_minInterval + nInteral, subImg2.shape[1] - nInteral)]
right_edge = subImg2.shape[1] - cols_minInterval + nInteral + np.argmin(right_scope_diff)
"""



# !/usr/bin/env python
# -- coding: utf-8 --
# @Author zengxiaohui
# Datatime:4/29/2021 4:22 PM
# @File:common
import os
import shutil


def resetDir(dirpath):
    """判断文件夹是否存在存在那么删除重建，不存在那么创建"""
    if not os.path.exists(dirpath):
        os.makedirs(dirpath)
    else:
        shutil.rmtree(dirpath)
        os.makedirs(dirpath)


def get_filelist(dir, Filelist):
    """获取文件夹及子文件夹下所有文件"""
    newDir = dir
    if os.path.isfile(dir):
        Filelist.append(dir)
        # # 若只是要返回文件文，使用这个
        # Filelist.append(os.path.basename(dir))
    elif os.path.isdir(dir):
        for s in os.listdir(dir):
            # 如果需要忽略某些文件夹，使用以下代码
            # if s == "xxx":
            # continue
            newDir = os.path.join(dir, s)
            get_filelist(newDir, Filelist)
    return Filelist



# !/usr/bin/env python
# -- coding: utf-8 --
# @Author zengxiaohui
# Datatime:4/29/2021 4:58 PM
# @File:image_utils
import cv2
import numpy as np
import base64
import re


def imgToBase64(imgpath):
    """将图片转base64"""
    with open(imgpath, 'rb') as f:
        base64_data = base64.b64encode(f.read())
        s = base64_data.decode()
    return s  # "data:image/jpeg;base64,{}".format(s)


def base64ToNparray(image_string):
    # 将base64图片转为numpy数组
    strinfo = re.compile(r'^data:image/\w+;base64,')
    image_string = strinfo.sub('', image_string)
    img_data = base64.b64decode(image_string)
    nparr = np.fromstring(img_data, np.uint8)
    img_np = cv2.imdecode(nparr, cv2.IMREAD_COLOR)
    return img_np


def opencvToBase64(imgCv):
    """opencv格式换位base64"""
    image = cv2.imencode('.jpg', imgCv)[1]
    base64_data = str(base64.b64encode(image))[2:-1]
    return base64_data



# !/usr/bin/env python
# -- coding: utf-8 --
# @Author zengxiaohui
# Datatime:4/29/2021 8:41 PM
# @File:time_utils
import time


def get_time_stamp():
    """获取毫秒级的时间"""
    ct = time.time()
    local_time = time.localtime(ct)
    data_head = time.strftime("%Y-%m-%d %H:%M:%S", local_time)
    data_secs = (ct - int(ct)) * 1000
    time_stamp = "%s.%03d" % (data_head, data_secs)
    return time_stamp



# !/usr/bin/env python
# -- coding: utf-8 --
# @Author zengxiaohui
# Datatime:4/29/2021 8:35 PM
# @File:data_utils
def getkeys_by_tuple(tuplemm):
    """返回元组中所有的key"""
    return [tuple_tmp[0] for tuple_tmp in tuplemm]


def tuple_value_by_key(tuplemm, key):
    """
    根据key获取元组中value的值
    """
    for tuple_tmp in tuplemm:
        if tuple_tmp[0] == key:
            return tuple_tmp[1]


def tuple_key_by_value(tuplemm, value):
    """
    根据value获取元组中key的值
    """
    for tuple_tmp in tuplemm:
        if tuple_tmp[1] == value:
            return tuple_tmp[0]



# !/usr/bin/env python
# -- coding: utf-8 --
# @Author zengxiaohui
# Datatime:4/29/2021 8:38 PM
# @File:obj_utils
from zengxh_py_tools.python.string_utils import str_is_null


def obj_is_null(obj):
    """判断对象是否为空"""
    if obj is None:
        return True
    if isinstance(obj, list) and len(obj) == 0:
        return True
    if isinstance(obj, str):
        return str_is_null(obj)



# !/usr/bin/env python
# -- coding: utf-8 --
# @Author zengxiaohui
# Datatime:4/29/2021 4:22 PM
# @File:xml_utils
from xml.dom.minidom import parse
from lxml.etree import Element, SubElement, tostring


def read_predict_xml(label_path):
    # 读取xml中的内容
    gt_lists = []
    # 获取所有xml中object对象
    small_dom_tree = parse(label_path)
    small_root_node = small_dom_tree.documentElement
    objects = small_root_node.getElementsByTagName('object')

    for obj in objects:
        name = obj.getElementsByTagName('name')[0].childNodes[0].nodeValue
        if name in ["cell", "barcode"]:
            continue
        xmin = obj.getElementsByTagName('xmin')[0].childNodes[0].nodeValue
        ymin = obj.getElementsByTagName('ymin')[0].childNodes[0].nodeValue
        xmax = obj.getElementsByTagName('xmax')[0].childNodes[0].nodeValue
        ymax = obj.getElementsByTagName('ymax')[0].childNodes[0].nodeValue
        gt = {"coordinate": "{},{},{},{}".format(xmin, ymin, xmax, ymax),
              "col_idx": "",
              "row_idx": "",
              "prob": "",
              "tag": name}
        gt_lists.append(gt)
        #
        #     pt_x1, pt_y1, pt_x2, pt_y2 = pt["coordinate"].split(",")
        #     pt_col, pt_row = pt["col_idx"], pt["row_idx"]
        #     pt_label = pt["tag"]
    return gt_lists


def json_2_xml_save(save_xml_path, jsons, origin_imgname):
    # 将jsons保存为xml
    node_root = Element('annotation')

    node_folder = SubElement(node_root, 'folder')
    node_folder.text = 'images'
    node_filename = SubElement(node_root, 'filename')
    node_filename.text = origin_imgname

    node_size = SubElement(node_root, 'size')
    node_width = SubElement(node_size, 'width')
    node_width.text = str(0)

    node_height = SubElement(node_size, 'height')
    node_height.text = str(0)

    node_depth = SubElement(node_size, 'depth')
    node_depth.text = '3'

    for box in jsons:
        node_object = SubElement(node_root, 'object')
        node_name = SubElement(node_object, 'name')
        node_name.text = box['tag']
        node_difficult = SubElement(node_object, 'difficult')
        node_difficult.text = "0"
        node_bndbox = SubElement(node_object, 'bndbox')

        coord_list = list(map(lambda x: int(float(x)), box['coordinate'].split(',')))

        node_xmin = SubElement(node_bndbox, 'xmin')
        node_xmin.text = str(coord_list[0])
        node_ymin = SubElement(node_bndbox, 'ymin')
        node_ymin.text = str(coord_list[1])
        node_xmax = SubElement(node_bndbox, 'xmax')
        node_xmax.text = str(coord_list[2])
        node_ymax = SubElement(node_bndbox, 'ymax')
        node_ymax.text = str(coord_list[3])

    xml = tostring(node_root, pretty_print=True)  # 格式化显示，该换行的换行
    with open(save_xml_path, 'wb') as f:
        f.write(xml)



# !/usr/bin/env python
# -- coding: utf-8 --
# @Author zengxiaohui
# Datatime:4/29/2021 8:36 PM
# @File:string_utils
def str_is_null(strings):
    """判断字符串是否为空"""
    if strings is None:
        return True
    if strings.strip() == '':
        return True  # 是空
    return False



# !/usr/bin/env python
# -- coding: utf-8 --
# @Author zengxiaohui
# Datatime:4/29/2021 4:27 PM
# @File:__init__.py
from .obj_utils import *
from .string_utils import *
from .time_utils import *
from .tuple_utils import *



# !/usr/bin/env python
# -- coding: utf-8 --
# @Author zengxiaohui
# Datatime:4/29/2021 4:27 PM
# @File:__init__.py
from .common import *
from .image_utils import *
from .txt_utils import *
from .xml_utils import *



# !/usr/bin/env python
# -- coding: utf-8 --
# @Author zengxiaohui
# Datatime:4/29/2021 8:39 PM
# @File:web.py
import os

import requests
import socket
from xlrd import open_workbook
from xlutils.copy import copy

def getiphostname():
    # 获取计算机名称
    hostname = socket.gethostname()
    # 获取本机IP
    ip = socket.gethostbyname(hostname)
    return ip, hostname


def post_get_data(url='http://127.0.0.1:8000/pv/api/', data={"imageName": "sdfsdfasd"}):
    """发送post请求通过表单列的方式并获取数据"""
    r = requests.post(url, data=data)
    return r.text


def post_get_data_by_json(url='http://127.0.0.1:8000/pv/api/', json={"imageName": "sdfsdfasd"}):
    """发送post请求通过json并获取数据"""
    r = requests.post(url, json=json)
    return r.text


def get_request_data(url):
    """get请求获取数据"""
    r = requests.get(url)
    return r.text

def export_excel(values=["1", "2", "3"]):
    """读取xls模板并写入数据"""
    export_excel_dir = os.path.join("","")
    if not os.path.exists(export_excel_dir):
        os.makedirs(export_excel_dir)
    filename = "统计报表.xls"
    xlstemplates = os.path.join(export_excel_dir, 'tongji.xls')
    reportSavePath = os.path.join(export_excel_dir, filename)

    rexcel = open_workbook(xlstemplates, formatting_info=True)  # 用wlrd提供的方法读取一个excel文件 打开excel，保留文件格式
    rows = rexcel.sheets()[0].nrows  # 用wlrd提供的方法获得现在已有的行数
    excel = copy(rexcel)  # 用xlutils提供的copy方法将xlrd的对象转化为xlwt的对象
    table = excel.get_sheet(0)  # 用xlwt对象的方法获得要操作的sheet

    row = rows
    for value in values:
        table.write(row, 0, value)  # xlwt对象的写方法，参数分别是行、列、值
        table.write(row, 1, "haha")
        table.write(row, 2, "lala")
        row += 1
    excel.save(reportSavePath)  # xlwt对象的保存方法，这时便覆盖掉了原来的excel
    return reportSavePath, filename


# !/usr/bin/env python
# -- coding: utf-8 --
# @Author zengxiaohui
# Datatime:4/29/2021 2:41 PM
# @File:setup
import os

from setuptools import setup, find_packages

here = os.path.abspath(os.path.dirname(__file__))
with open(os.path.join(here, 'README.md'),encoding="utf-8") as f:
    README = f.read()


setup(
    author='zengxiaohui', #可以用来指定该package的作者信息。
    name='python_developer_tools', # 是该package的名字。该名字可以由字母，数字，_和-组成。并且这个名字不能与其他已经上传至pypi.org的项目相同
    version='0.0.2', #是当前package的版本。关于版本的详细信息请参考
    author_email="2362651588@qq.com", #可以用来指定该package的作者信息。
    description='zengxiaohui python tools',
    long_description=README,
    long_description_content_type='text/markdown',
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "Intended Audience :: System Administrators",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: Microsoft :: Windows",
        "Operating System :: POSIX",
        "Programming Language :: Python",
        "Topic :: System :: Networking :: Monitoring",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
    ],
    url='https://github.com/carlsummer/zengxh_py_tools',
    packages=find_packages(),
    install_requires=["numpy","scipy","matplotlib"],

)


# !/usr/bin/env python
# -- coding: utf-8 --
# @Author zengxiaohui
# Datatime:4/29/2021 4:27 PM
# @File:common
import numpy as np

def computeIOU(pred_loc, ground_info):
    # 计算2个框的iou
    xmin1, ymin1, xmax1, ymax1 = [round(float(val)) for val in pred_loc]
    xmin2, ymin2, xmax2, ymax2 = [round(float(val)) for val in ground_info]

    # 求交集部分左上角的点
    xmin = max(xmin1, xmin2)
    ymin = max(ymin1, ymin2)
    # 求交集部分右下角的点
    xmax = min(xmax1, xmax2)
    ymax = min(ymax1, ymax2)
    # 计算输入的两个矩形的面积
    s1 = (xmax1 - xmin1) * (ymax1 - ymin1)
    s2 = (xmax2 - xmin2) * (ymax2 - ymin2)

    # 计算总面积
    s = s1 + s2
    # 计算交集
    inter_area = (xmax - xmin) * (ymax - ymin)
    iou = inter_area / (s - inter_area)
    return round(iou, 2)

def bbox_expand(bbox, bbox_scale):
    # 对bbox进行缩放
    assert bbox_scale >= 0
    xmin, ymin, xmax, ymax = bbox
    h = ymax - ymin + 1
    new_h = h * bbox_scale
    w = xmax - xmin + 1
    new_w = w * bbox_scale
    cx = 0.5 * (xmin + xmax)
    cy = 0.5 * (ymin + ymax)
    xmin = cx - 0.5 * new_w
    xmax = cx + 0.5 * new_w
    ymin = cy - 0.5 * new_h
    ymax = cy + 0.5 * new_h
    return np.array([xmin, ymin, xmax, ymax])


# !/usr/bin/env python
# -- coding: utf-8 --
# @Author zengxiaohui
# Datatime:4/29/2021 4:18 PM
# @File:txt_utils
import ast


def save_txt(json, save_path):
    # 将内容保存为txt
    if json is not None:
        with open(save_path, 'a+', encoding='utf-8') as f:
            f.write(str(json))


def read_predict_txt(txt_path):
    # 读取txt中的内容
    with open(txt_path, 'r') as f:
        file_context = f.read()
    predict_dict = ast.literal_eval(file_context)
    return predict_dict
